;***************************************************************************************;
;											;
;	Routine per la scrittura/verifica della flash in run-time			;
;	SPECIFICHE PER "PIC18FXXX"							;
;	Versione 1.1		     		Ult.agg. 24/03/06			;
;											;
;***************************************************************************************;


;--- REGISTRI DA DEFINIRE NEL MAIN PROGRAM!  -------------------;
;	BYTE_NUM, FLASH_COUNTER					; 
;	FLASH_ADDR_U, FLASH_ADDR_H, FLASH_ADDR_L		; FLASH POINTER
;	FLASH_BUFF						; ADDR 1ST OF "BYTE_NUM" BYTE!!!
;---------------------------------------------------------------;

;---------------------------------------------------------------;
;								;
;	WRITE_FLASH						;
;								;
;---------------------------------------------------------------;
;	NOTE FOR WRITE_FLASH:					;
;	-> 8 BYTE WRITING					;
;	-> TBLPTR MUST BE PROPERLY SET				;
; 	-> GIE IS SET BY ROUTINE				;
;	-> INDF0 WAS USED					;
;	-> LFSR0 MUST BE PROPERLY SET				;
;---------------------------------------------------------------;


WRITE_8_BYTE:
	MOVLW	D'8' 			; number of bytes in holding register
	MOVWF	FLASH_COUNTER		;

WRITE_BYTES2:				;
	MOVF	POSTINC0, W		; get low byte of buffer data
	MOVWF	TABLAT			; present data to table latch
	TBLWT*+				; write data, perform a short write
					; to internal TBLWT holding register.
	DECFSZ	FLASH_COUNTER		; loop until buffers are full
	BRA	WRITE_BYTES2		;

	TBLRD*-				;<<<<<<<<<<<<<< read data, perform a short write

	BSF	EECON1,EEPGD		; point to FLASH program memory
	BCF	EECON1,CFGS		; access FLASH program memory
	BSF	EECON1,WREN		; enable write to memory
	BCF	INTCON,GIE		; disable interrupts
	MOVLW	0x55			;
	MOVWF	EECON2			; write 55h
	MOVLW	0xAA			;
	MOVWF	EECON2			; write AAh
	BSF	EECON1, WR		; start program (CPU stall)
;	BSF	INTCON, GIE		; re-enable interrupts

	BCF	EECON1,WREN		; disable write to memory
	RETURN


;---------------------------------------------------------------;
;								;
;	ERASE_FLASH						;
;								;
;---------------------------------------------------------------;
;	NOTE FOR ERASE_FLASH:					;
;	-> BYTE_NUM USED FOR CONSECUTIVE BLOCK ERASING!		;
; 	-> GIE IS SET BY ROUTINE				;
;---------------------------------------------------------------;

ERASE_FLASH:
	MOVF	FLASH_ADDR_U, W		; SETTING FLASH POINTER
	MOVWF	TBLPTRU			; 
	MOVF	FLASH_ADDR_H, W		; SETTING FLASH POINTER
	MOVWF	TBLPTRH			;
	MOVF	FLASH_ADDR_L, W		; SETTING FLASH POINTER
	MOVWF	TBLPTRL			;

ERASE_FLASH_NO_ADDR:
	MOVF	BYTE_NUM, W		; number of bytes in holding register
	MOVWF	FLASH_COUNTER		;

ERASE_ROW
	MOVF	TBLPTRU, W		; SETTING FLASH POINTER
	MOVWF	FLASH_ADDR_U		;
	MOVF	TBLPTRH, W		; SETTING FLASH POINTER
	MOVWF	FLASH_ADDR_H		;
	MOVF	TBLPTRL, W		; SETTING FLASH POINTER
	MOVWF	FLASH_ADDR_L		;

	BSF	EECON1,EEPGD		; point to FLASH program memory
	BCF	EECON1,CFGS		; access FLASH program memory
	BSF	EECON1,WREN		; enable write to memory
	BSF	EECON1,FREE		; enable Row Erase operation
	BCF	INTCON,GIE		; disable interrupts
	MOVLW	0x55
	MOVWF	EECON2			; write 55h
	MOVLW	0xAA
	MOVWF	EECON2			; write AAh
	BSF	EECON1,WR		; start erase (CPU stall)
;	BSF	INTCON,GIE 		; re-enable interrupts

	MOVLW	D'64'			;; ADD 64 TO ADDR.
	ADDWF	TBLPTRL, F		;;
	MOVLW	0x00			;;
	ADDWFC	TBLPTRH, F		;;
	ADDWFC	TBLPTRU, F		;;

	DECFSZ	BYTE_NUM, F		; loop until done
	BRA	ERASE_ROW
	RETURN


;-----------------------------------------------------------------------------

